package com.pj.current.netty.command.handler.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.IHandler;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.netty.dto.OKMsg;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.task.TaskService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_filing.task.Base64ToImage;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 上传人员信息
 */
@Service
@Slf4j
public class UploadUserHandler implements IHandler {
    @Resource
    private TbTerminalService tbTerminalService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private TaskService taskService;

    @Resource
    private TbDeptService tbDeptService;



    @Override
    public CommandType commandType() {
        return CommandType.UPLOAD_USER;
    }

    @Override
    public void handler(ChannelHandlerContext ctx, RequestPackageDTO packageDTO) {
        JSONObject params = JSONUtil.parseObj(packageDTO.getPayload().getParams());
        String sn = params.getStr("sn");
        TbTerminal tbTerminal = tbTerminalService.findBySN(sn);
        if (tbTerminal == null ) {
            log.error("========设备不存在=========:{}", sn);
            return;
        }
        ctx.write(OKMsg.ok());
        ctx.flush();
        WhitelistDTO uploadUserDTO = JSONUtil.toBean(params.getStr("users"), WhitelistDTO.class);
        TbPersonFiling personFiling = tbPersonFilingService.findByIdCard(uploadUserDTO.getIdentity_number());
        if (personFiling == null) {
            TbPersonFiling   tbPersonFiling = new TbPersonFiling();
           List<TbDept>deptList= tbDeptService.list();
           deptList.stream().filter(tbDept -> tbDept.getPassType()==TbDept.PassTypeEnum.BLOW.getType()).findAny()
                   .ifPresent(tbDept -> tbPersonFiling.setDeptId(tbDept.getId()));
            Date now = new Date();
            tbPersonFiling.setCreateTime(now)
                    .setUpdateTime(now)
                    .setPin(uploadUserDTO.getPin())
                    .setCreateBy("设备上传")
                    .setIdCard(uploadUserDTO.getIdentity_number())
                    .setJudgeState(1).setJudgeTime(now).setJudgeBy("终端备案上传")
                    .setName(uploadUserDTO.getName());
            tbPersonFilingService.save(tbPersonFiling);
            //转换图片
            taskService.addTask(new Base64ToImage(RandomUtil.randomNumbers(10),2000,tbPersonFiling.getId(),uploadUserDTO.getPhoto()));
        }
        List<TbTerminal> list = tbTerminalService.list();
        //分发人员信息,不分发上传设备
        list.stream().filter(t -> !StrUtil.equals(t.getSn(), sn))
                .forEach(t -> {
                    tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, t);
                    tbTerminalService.startHeartCommand(sn);
                });
    }
}
