package com.pj.project.tb_car_filing;

import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.tb_car_black.TbCarBlack;
import com.pj.project.tb_car_black.TbCarBlackService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project4sp.spcfg.SpCfgUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_car_filing -- 车辆备案
 *
 * @author qzyReal
 */
@Service
public class TbCarFilingService extends ServiceImpl<TbCarFilingMapper, TbCarFiling> implements IService<TbCarFiling> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbCarFilingMapper tbCarFilingMapper;

    @Resource
    private TbDeptService tbDeptService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private TbCarBlackService tbCarBlackService;


    /**
     * 增
     */
    public void add(TbCarFiling t) {
        String carNo=t.getCarNo().toUpperCase();
        String idCarNo = t.getDriverIdCard();
        //原来是否录入过
        TbCarFiling checkFiling = this.findByDriverIdCar(idCarNo);
        if (checkFiling != null) {
            throw new BusinessException("司机已备案其他车辆");
        }
        TbCarFiling carNoCheckFiling=this.findByCarNo(carNo);
        if (carNoCheckFiling!=null){
            throw new BusinessException("车辆已备案");
        }
        String needJudge = SpCfgUtil.getFillingJudge();
        if (StrUtil.equals(needJudge, "2")) {
            t.setJudgeState(1).setJudgeTime(new Date()).setJudgeBy("系统自动审核");
        }
        t.setCreateBy(StpUtil.getLoginIdAsString()).setCarNo(carNo)
                .setCreateTime(new Date());
        handler(t);
        this.handlerDriver(t);
        this.save(t);
    }

    /**
     * @param tbCarFiling
     */
    private void handlerDriver(TbCarFiling tbCarFiling) {
        String idCarNo = tbCarFiling.getDriverIdCard();
        //人员是否备案
        TbPersonFiling t = tbPersonFilingService.findByIdCard(idCarNo);
        Date now = new Date();
        if (t == null) {
            String pin = RandomUtil.randomNumbers(9);
            tbCarFiling.setDriverPin(pin);
            t = new TbPersonFiling();
            t.setDeptId(tbCarFiling.getDeptId()).setPhoto(tbCarFiling.getDriverPhoto()).setPin(pin)
                    .setIdCard(tbCarFiling.getDriverIdCard()).setName(tbCarFiling.getDriverName())
                    .setPhone(tbCarFiling.getDriverPhoto()).setPin(tbCarFiling.getDriverPin());
            String needJudge = SpCfgUtil.getFillingJudge();
            if (StrUtil.equals(needJudge, "2")) {
                t.setJudgeState(1).setJudgeTime(now).setJudgeBy("系统自动审核");
            }
            //下发司机白名单
            tbPersonFilingService.issued(t);
        } else {
            tbCarFiling.setDriverPin(t.getPin());
        }
    }

    /**
     * 一个司机只能备案一次
     *
     * @param idCarNo
     * @return
     */
    public TbCarFiling findByDriverIdCar(String idCarNo) {
        QueryWrapper<TbCarFiling> ew = new QueryWrapper<>();
        ew.eq("driver_id_card", idCarNo);
        return getOne(ew);
    }

    private void handler(TbCarFiling t) {
        if (!IdcardUtil.isValidCard(t.getDriverIdCard())) {
           // throw new BusinessException("身份证不正确");
        }
        TbDept tbDept = tbDeptService.getById(t.getDeptId());
        if (tbDept == null) {
            throw new BusinessException("请选择组织");
        }
        String carNo = t.getCarNo().toUpperCase();
        Date now = new Date();
        t.setDeptName(tbDept.getName()).setUpdateTime(now).setCarNo(carNo)
                .setUpdateBy(StpUtil.getLoginIdAsString());
        t.setCarNo(t.getCarNo().toUpperCase());
    }

    /**
     * 删
     */
    public void delete(Long id) {
        TbCarFiling db = this.getById(id);
        String idCarNo = db.getDriverIdCard();
        TbPersonFiling tbPersonFiling = tbPersonFilingService.findByIdCard(idCarNo);
        //如果不存在白名单，则
        if (tbPersonFiling == null) {
            //删除设备的白名单
            tbPersonFilingService.deleteTerminalWhilte(db.getDriverPin());
        }
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbCarFiling t) {
        handler(t);
        String carNo=t.getCarNo();
        TbCarFiling carNoCheck=this.findByCarNo(carNo);
        if (carNoCheck!=null&&t.getId().longValue()!=carNoCheck.getId().longValue()){
            throw new BusinessException("车辆已备案");
        }
        TbCarFiling db = this.getById(t.getId());
        String idCarNo = t.getDriverIdCard();
        //改了身份证
        if (!StrUtil.equals(idCarNo, db.getDriverIdCard())) {
            //原来是否录入过
            TbCarFiling checkFiling = this.findByDriverIdCar(idCarNo);
            if (checkFiling != null) {
                throw new BusinessException("司机已备案其他车辆");
            }
            //司机身份证不一样 删除旧的，下发新的
            tbPersonFilingService.deleteTerminalWhilte(db.getDriverPin());
            this.handlerDriver(t);
        }
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbCarFiling getById(Long id) {
        return tbCarFilingMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbCarFiling> getList(SoMap so) {
        return tbCarFilingMapper.getList(so);
    }


    public TbCarFiling findByCarNo(String carNo) {
        QueryWrapper<TbCarFiling> ew = new QueryWrapper<>();
        ew.eq("car_no", carNo);
        List<TbCarFiling> list = list(ew);
        return list.isEmpty() ? null : list.get(0);
    }

    public void judge(TbCarFiling t) {
        t.setJudgeTime(new Date()).setJudgeBy(StpUserUtil.getAdminName());
        this.updateById(t);
    }

    public List<TbCarFiling> getForWx(SoMap soMap) {

        return tbCarFilingMapper.getForWx(soMap);
    }
}
