package com.pj.project.wx;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.net.URLDecoder;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;

import com.pj.current.config.MyConfig;

import com.pj.current.config.WxConfig;
import com.pj.project.wx.bo.BaseTemplate;
import com.pj.project.wx.bo.MsgDataBO;

import com.pj.utils.cache.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class WxService {

    @Resource
    WxConfig wxConfig;
    public String getRedirectUrl(String path, String state) {
        String redirectUrl = wxConfig.getDomain() + path;
        String encoderUrl = URLDecoder.decode(redirectUrl, Charset.forName("utf-8"));
        String url = wxConfig.getAuthLoginUrl().replace("REDIRECT_URI", encoderUrl);
        if (StrUtil.isNotEmpty(state)) {
            url = url.replace("STATE", state);
        }
        log.info("get redirect url:{}", url);
        return url;
    }

    public String getOpenidByCode(String code, String openid) {
        String openidUrl = wxConfig.getOpenidUrl().replace("CODE", code);
        String resp = HttpUtil.get(openidUrl);
        JSONObject result = JSONUtil.parseObj(resp);
        log.info("get openid result:{}", resp);
        String newOpenid = result.getStr("openid");
        if (StrUtil.isNotEmpty(newOpenid)) {
            return newOpenid;
        }
        return openid;
    }

    public SortedMap getWxConfig(String url) {
        Long nowTime = System.currentTimeMillis() / 1000;
        String jsTicket = RedisUtil.get(wxConfig.getJsApiTicketKey());
        SortedMap<String, Object> params = new TreeMap<>();
        params.put("noncestr", RandomUtil.randomString(32));
        params.put("jsapi_ticket", jsTicket);
        params.put("timestamp", nowTime);
        params.put("url", url);
        String sign = SecureUtil.sha1(params.toString().substring(1, params.toString().lastIndexOf("}")).replaceAll(", ", "&"));
        params.put("sign", sign);
        params.put("appId", wxConfig.getAppId());
        return params;
    }

    @Async
    public void sendTemplateMsg(String templateId, String openid, MsgDataBO data) {
        String accessToken = RedisUtil.get(wxConfig.getAccessTokenKey());
        BaseTemplate baseTemplate = new BaseTemplate(openid, templateId, data);
        String json = JSON.toJSONString(baseTemplate);
        String url = wxConfig.getSendMsgUrl().replace("ACCESS_TOKEN", accessToken);
        String resp = HttpUtil.post(url, json);
        log.info("send wx msg{},{},{};return :{}", templateId, openid, JSONUtil.toJsonStr(data), resp);
    }

    @Async
    public void sendTemplateMsg(String templateId, String openid, Object data, String detailUrl) {
        if (StrUtil.isEmpty(openid)){
            return;
        }
        String accessToken = RedisUtil.get(wxConfig.getAccessTokenKey());
        BaseTemplate baseTemplate = new BaseTemplate(openid, templateId, detailUrl, data);
        String json = JSON.toJSONString(baseTemplate);
        String url = wxConfig.getSendMsgUrl().replace("ACCESS_TOKEN", accessToken);
        String resp = HttpUtil.post(url, json);
        JSONObject result = JSONUtil.parseObj(resp);
        int code = result.getInt("errcode");
        log.info("send wx msg{},{},{};return :{}", templateId, openid, JSONUtil.toJsonStr(data), resp);

    }

}
